package nl.iobyte.themepark.api.attraction.objects;

import com.cryptomorin.xseries.XMaterial;
import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.config.objects.Configuration;
import nl.iobyte.themepark.api.event.region.RegionMaterialChangeEvent;
import nl.iobyte.themepark.api.event.region.RegionNameChangeEvent;
import org.bukkit.Material;

import java.util.LinkedHashMap;
import java.util.Optional;

public class Region {

    private final String id;
    private String name;
    private transient Material material;
    private transient final Configuration configuration;
    private final LinkedHashMap<String, Attraction> attractions = new LinkedHashMap<>();

    public Region(String id, String name) {
        this.id = id;
        this.name = name;
        this.material = XMaterial.NAME_TAG.parseMaterial();
        configuration = new Configuration(ThemePark.getInstance(), "attractions/"+id+".yml", false);
    }

    public Region(String id, String name, Material material) {
        this.id = id;
        this.name = name;
        this.material = Optional.ofNullable(material).orElse(XMaterial.NAME_TAG.parseMaterial());
        configuration = new Configuration(ThemePark.getInstance(), "attractions/"+id+".yml", false);
    }

    /**
     * Get ID of Region
     * @return String
     */
    public String getID() {
        return id;
    }

    /**
     * Get Name of Region
     * @return String
     */
    public String getName() {
        return name;
    }

    /**
     * Set Name of Region
     * @param name String
     */
    public void setName(String name) {
        String old = this.name;
        this.name = name;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new RegionNameChangeEvent(
                this,
                old,
                name
        ));
    }

    /**
     * Get Material of Region
     * @return Material
     */
    public Material getMaterial() {
        return material;
    }

    /**
     * Set Material of Region
     * @param material Material
     */
    public void setMaterial(Material material) {
        Material old = this.material;
        this.material = material;
        ThemePark.getInstance().getAPI().getEventDispatcher().call(new RegionMaterialChangeEvent(
                this,
                old,
                material
        ));
    }

    /**
     * Get Map with Attractions
     * @return LinkedHashMap<String, Attraction>
     */
    public LinkedHashMap<String, Attraction> getAttractions() {
        return attractions;
    }

    /**
     * Get Region Configuration file
     * @return Configuration
     */
    public Configuration getConfiguration() {
        return configuration;
    }

    /**
     * Add Attraction to Region
     * @param attraction Attraction
     */
    public void addAttraction(Attraction attraction) {
        if(attraction.getID() == null || attraction.getID().isEmpty())
            return;

        if(hasAttraction(attraction.getID()))
            return;

        attractions.put(attraction.getID(), attraction);
    }

    /**
     * Has Attraction with ID
     * @param id String
     * @return Attraction
     */
    public boolean hasAttraction(String id) {
        if(id == null || id.isEmpty())
            return false;

        return attractions.containsKey(id);
    }

    /**
     * Get Attraction by ID from Region
     * @param id String
     * @return Attraction
     */
    public Attraction getAttraction(String id) {
        if(id == null || id.isEmpty())
            return null;

        return attractions.get(id);
    }

    /**
     * Remove Attraction by ID from Region
     * @param id String
     * @return Attraction
     */
    public Attraction removeAttraction(String id) {
        return attractions.remove(id);
    }

}
